@@ -22,25 +22,20 @@
 #else
 #undef errno
 #define errno GetLastError()
-typedef DWORD error_t;
+// FIXME: WLACH: seems to cause an error on mingw32 3.4.2.. is this needed?
+//typedef DWORD error_t;
 #undef EAGAIN
 #define EAGAIN WSAEWOULDBLOCK
 #endif
 
-static IWvStream *creator(WvStringParm s, IObject *obj, void *userdata)
+static IWvStream *creator(WvStringParm s)
 {
-    if (!obj)
-	obj = wvcreate<IWvStream>(s);
-    return new WvSSLStream(mutate<IWvStream>(obj),
-			   (WvX509Mgr *)userdata, 0, false);
+    return new WvSSLStream(wvcreate<IWvStream>(s), NULL, 0, false);
 }
 
-static IWvStream *screator(WvStringParm s, IObject *obj, void *userdata)
+static IWvStream *screator(WvStringParm s)
 {
-    if (!obj)
-	obj = wvcreate<IWvStream>(s);
-    return new WvSSLStream(mutate<IWvStream>(obj),
-			   (WvX509Mgr *)userdata, 0, true);
+    return new WvSSLStream(wvcreate<IWvStream>(s), NULL, 0, true);
 }
 
 static WvMoniker<IWvStream> reg("ssl", creator);
@@ -56,18 +51,22 @@ static int wv_verify_cb(int preverify_ok, X509_STORE_CTX *ctx)
    return 1;
 }
 
-WvSSLStream::WvSSLStream(IWvStream *_slave, WvX509Mgr *x509,
+WvSSLStream::WvSSLStream(IWvStream *_slave, WvX509Mgr *_x509,
     WvSSLValidateCallback _vcb, bool _is_server) :
     WvStreamClone(_slave), debug("WvSSLStream", WvLog::Debug5),
     write_bouncebuf(MAX_BOUNCE_AMOUNT), write_eat(0),
     read_bouncebuf(MAX_BOUNCE_AMOUNT), read_pending(false)
 {
+    x509 = _x509;
+    if (x509)
+	x509->addRef(); // openssl may keep a pointer to this object
+    
     vcb = _vcb;
     is_server = _is_server;
     ctx = NULL;
     ssl = NULL;
     meth = NULL;
-    sslconnected = false;
+    sslconnected = ssl_stop_read = ssl_stop_write = false;
     
     wvssl_init();
     
@@ -136,6 +135,8 @@ WvSSLStream::WvSSLStream(IWvStream *_slave, WvX509Mgr *x509,
         }
     }
     
+    //SSL_CTX_set_read_ahead(ctx, 1);
+
     ERR_clear_error();
     ssl = SSL_new(ctx);
     if (!ssl)
@@ -148,6 +149,8 @@ WvSSLStream::WvSSLStream(IWvStream *_slave, WvX509Mgr *x509,
 	SSL_set_verify(ssl, SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE, 
                        wv_verify_cb);
 
+
+
     debug("SSL stream initialized.\n");
 
     // make sure we run the SSL_connect once, after our stream is writable
@@ -159,10 +162,11 @@ WvSSLStream::~WvSSLStream()
 {
     close();
     
-    debug("Shutting down SSL connection.\n");
+    debug("Deleting SSL connection.\n");
     if (geterr())
 	debug("Error was: %s\n", errstr());
     
+    WVRELEASE(x509);
     wvssl_free();
 }
 
@@ -213,6 +217,12 @@ size_t WvSSLStream::uread(void *buf, size_t len)
                 break;
 	    }
             buf = (unsigned char *)buf + amount;
+	    
+	    // FIXME: this shouldn't be necessary, but it resolves weird
+	    // problems when the other end disconnects in the middle of
+	    // SSL negotiation, but only on emakela's machine.  I don't
+	    // know why.  -- apenwarr (2004/02/10)
+	    break;
         }
 
         // attempt to read
@@ -222,27 +232,32 @@ size_t WvSSLStream::uread(void *buf, size_t len)
         
 	ERR_clear_error();
         int result = SSL_read(ssl, data, avail);
+	// debug("<< SSL_read result %s for %s bytes (wanted %s)\n",
+	//      result, avail, len);
         if (result <= 0)
         {
 	    error_t err = errno;
             read_bouncebuf.unalloc(avail);
-            int errcode = SSL_get_error(ssl, result);
-            switch (errcode)
+            int sslerrcode = SSL_get_error(ssl, result);
+	    switch (sslerrcode)
             {
                 case SSL_ERROR_WANT_READ:
+		    debug("<< SSL_read() needs to wait for writable.\n");
+                    break; // wait for later
                 case SSL_ERROR_WANT_WRITE:
-//                    debug("<< SSL_read() needs to wait for readable.\n");
+		    debug("<< SSL_read() needs to wait for readable.\n");
                     break; // wait for later
                     
                 case SSL_ERROR_NONE:
                     break; // no error, but can't make progress
                     
-                case SSL_ERROR_ZERO_RETURN:
+	        case SSL_ERROR_ZERO_RETURN:
 		    debug("<< EOF: zero return\n");
-		    // signal that we want to be closed
-		    // after we return our buffer
-		    autoclose_time = time(NULL);
-		    noread(); // EOF
+		
+		    // don't do this if we're returning nonzero!
+		    // (SSL has no way to do a one-way shutdown, so if SSL
+		    // detects a read problem, it's also a write problem.)
+		    if (!total) { noread(); nowrite(); }
                     break;
 
 		case SSL_ERROR_SYSCALL:
@@ -250,19 +265,30 @@ size_t WvSSLStream::uread(void *buf, size_t len)
 		    {
 			if (result == 0)
 			{
-			    debug("<< EOF: syscall error\n");
-			    // signal that we want to be closed
-			    // after we return our buffer
-			    autoclose_time = time(NULL);
-			    noread();
+			    debug("<< EOF: syscall error "
+				  "(%s/%s, %s/%s) total=%s\n",
+				  stop_read, stop_write,
+				  isok(), cloned && cloned->isok(), total);
+			    
+			    // don't do this if we're returning nonzero!
+			    // (SSL has no way to do a one-way shutdown, so
+			    // if SSL detects a read problem, it's also a
+			    // write problem.)
+			    if (!total) { noread(); nowrite(); }
 			}
-	                break; 
 		    }
-		    debug("<< SSL_read() %s\n", strerror(errno));
+		    else
+		    {
+			debug("<< SSL_read() err=%s (%s)\n",
+			    err, strerror(err));
+			seterr_both(err, WvString("SSL read: %s",
+			    strerror(err)));
+		    }
+		    break;
                     
                 default:
                     printerr("SSL_read");
-                    seterr("SSL read error #%s", errcode);
+                    seterr("SSL read error #%s", sslerrcode);
                     break;
             }
             read_pending = false;
@@ -275,7 +301,8 @@ size_t WvSSLStream::uread(void *buf, size_t len)
         read_bouncebuf.unalloc(avail - result);
     }
 
-    // debug("<< read %s bytes\n", total);
+    // debug("<< read %s bytes (%s, %s)\n",
+    //	  total, isok(), cloned && cloned->isok());
     return total;
 }
 
@@ -332,18 +359,22 @@ size_t WvSSLStream::uwrite(const void *buf, size_t len)
         // attempt to write
         size_t used = write_bouncebuf.used();
         const unsigned char *data = write_bouncebuf.get(used);
-        
+
         ERR_clear_error();
         int result = SSL_write(ssl, data, used);
+	// debug("<< SSL_write result %s for %s bytes\n",
+	//      result, used);
         if (result <= 0)
         {
-            int errcode = SSL_get_error(ssl, result);
+            int sslerrcode = SSL_get_error(ssl, result);
             write_bouncebuf.unget(used);
-            switch (errcode)
+            switch (sslerrcode)
             {
                 case SSL_ERROR_WANT_READ:
+                    debug(">> SSL_write() needs to wait for readable.\n");
+                    break; // wait for later
                 case SSL_ERROR_WANT_WRITE:
-                    debug(">> SSL_write() needs to wait for writable.\n");
+                    // debug(">> SSL_write() needs to wait for writable.\n");
                     break; // wait for later
                     
 	        case SSL_ERROR_SYSCALL:
@@ -362,16 +393,19 @@ size_t WvSSLStream::uwrite(const void *buf, size_t len)
                     break; // no error, but can't make progress
                     
                 case SSL_ERROR_ZERO_RETURN:
+		    debug(">> SSL_write zero return: EOF\n");
                     close(); // EOF
                     break;
                     
                 default:
                     printerr("SSL_write");
-                    seterr(WvString("SSL write error #%s", errcode));
+                    seterr(WvString("SSL write error #%s", sslerrcode));
                     break;
             }
             break; // wait for next iteration
         }
+	else
+	    assert((size_t)result == used);
         write_bouncebuf.zap(); // force use of same position in buffer
         
         // locate next chunk to be written
@@ -392,12 +426,15 @@ size_t WvSSLStream::uwrite(const void *buf, size_t len)
         buf = (const unsigned char *)buf + size_t(result);
     }
     
-//    debug(">> wrote %s bytes\n", total);
+    //debug(">> wrote %s bytes\n", total);
     return total;
 }
 
 void WvSSLStream::close()
 {
+    debug("Closing SSL connection (ok=%s,sr=%s,sw=%s,child=%s).\n",
+	  isok(), stop_read, stop_write, cloned && cloned->isok());
+    
     if (ssl)
     {
         ERR_clear_error();
@@ -423,8 +460,37 @@ bool WvSSLStream::isok() const
 }
 
 
+void WvSSLStream::noread()
+{
+    // WARNING: openssl always needs two-way socket communications even for
+    // one-way encrypted communications, so we don't pass noread/nowrite
+    // along to the child stream.  This should be mostly okay, though,
+    // because we'll still send it close() once we have both noread() and
+    // nowrite().
+    ssl_stop_read = true;
+    if (ssl_stop_write)
+    {
+	WvStreamClone::nowrite();
+	WvStreamClone::noread();
+    }
+}
+
+
+void WvSSLStream::nowrite()
+{
+    // WARNING: see note in noread()
+    ssl_stop_write = true;
+    if (ssl_stop_read)
+    {
+	WvStreamClone::noread();
+	WvStreamClone::nowrite();
+    }
+}
+
+
 bool WvSSLStream::pre_select(SelectInfo &si)
 {
+    bool result = WvStreamClone::pre_select(si);
     // the SSL library might be keeping its own internal buffers
     // or we might have left buffered data behind deliberately
     if (si.wants.readable && (read_pending || read_bouncebuf.used()))
@@ -433,7 +499,15 @@ bool WvSSLStream::pre_select(SelectInfo &si)
 	return true;
     }
 
-    bool result = WvStreamClone::pre_select(si);
+    // if we're not ssl_connected yet, I can guarantee we're not actually
+    // writable, so don't ask WvStreamClone to wake up just because *he's*
+    // writable.
+    bool oldwr = si.wants.writable;
+    if (!sslconnected)
+	si.wants.writable = !!writecb;
+    result = WvStreamClone::pre_select(si);
+    si.wants.writable = oldwr;
+
 //    debug("in pre_select (%s)\n", result);
     return result;
 }
@@ -443,15 +517,16 @@ bool WvSSLStream::post_select(SelectInfo &si)
 {
     bool result = WvStreamClone::post_select(si);
 
-//    debug("in post_select (%s)\n", result);
-
     // SSL takes a few round trips to
     // initialize itself, and we mustn't block in the constructor, so keep
     // trying here... it is also turning into a rather cool place
     // to do the validation of the connection ;)
     if (!sslconnected && cloned && cloned->isok() && result)
     {
-//	debug("!sslconnected in post_select\n");
+	debug("!sslconnected in post_select (r=%s/%s, w=%s/%s, t=%s)\n",
+	    cloned->isreadable(), si.wants.readable,
+	    cloned->iswritable(), si.wants.writable,
+	    si.msec_timeout);
 	
 	undo_force_select(false, true, false);
 	
@@ -509,7 +584,7 @@ bool WvSSLStream::post_select(SelectInfo &si)
 		    else
 			seterr("Peer certificate is invalid!");
 	    	}
-		delete peercert;
+		WVRELEASE(peercert);
 	    }
 	    else
 	    {
