@@ -159,10 +159,11 @@ private:
     /** Prunes a branch of the watch tree. */
     void prune(UniWatchInfoTree *node);
     
-    /** Callback from UniMountTreeGen */
-    void gen_callback(const UniConfKey &key, WvStringParm value, void *userdata);
+    /** Callback from UniMountTreeGen (FIXME: that's a lie.) */
+    void gen_callback(const UniConfKey &key, WvStringParm value);
 
 protected:
+    friend class UniUnwrapGen;
     UniMountGen mounts;
     
 public:
