@@ -1,18 +1,24 @@
 /* -*- Mode: C++ -*-
  * Worldvisions Weaver Software:
  *   Copyright (C) 1997-2002 Net Integration Technologies, Inc.
- *
+ */
+/** \file
  * Provides support for forking processes.
  */
 
 #ifndef __WVFORK_H
 #define __WVFORK_H
 
+#ifndef _WIN32
 #include <unistd.h>
-#include "wvhashtable.h"
+#else
+typedef int pid_t;
+#endif
+
+#include "wvscatterhash.h"
 #include "wvcallback.h"
 
-DeclareWvTable(int);
+DeclareWvScatterTable(int);
 typedef WvCallback<void, pid_t> WvForkCallback;
 
 /**
