@@ -14,14 +14,18 @@
 
 #ifdef _WIN32
 #include <time.h>
-#include <Winsock2.h>
+#include <winsock2.h>
 #include <ws2tcpip.h>
+#include "wvwin32-sanitize.h"
 #else
 #include <unistd.h> // not strictly necessary, but EVERYBODY uses this...
 #include <sys/time.h>
 #endif
 
 
+// parameters are: owning-stream, userdata
+typedef WvCallback<void, WvStream&, void*> WvStreamCallback;
+
 /**
  * Unified support for streams, that is, sequences of bytes that may or
  * may not be ready for read/write at any given time.
@@ -33,12 +37,6 @@ class WvStream: public IWvStream
 {
     IMPLEMENT_IOBJECT(WvStream);
 public:
-    /**
-     * 'force' is the list of default SelectRequest values when you use the
-     * variant of select() that doesn't override them.
-     */
-    SelectRequest force;
-    
     /**
      * If this is set, select() doesn't return true for read unless the
      * given stream also returns true for write.
@@ -92,24 +90,6 @@ public:
     /** read a data block on the stream.  Returns the actual amount read. */
     virtual size_t read(void *buf, size_t count);
 
-    /**
-     * Read exactly count bytes from the stream.
-     *
-     * Notes:
-     *      must be using continue_select to use this function.
-     *      if timeout strikes or !isok() before count bytes could be read,
-     *          nothing is read and 0 is returned.
-     *      resets queuemin to 0.
-     * 
-     * FIXME: yes, that means if the stream closes, continue_read might not
-     * read the last bit of data.  You can use read() for that if you want.
-     */
-    virtual size_t continue_read(time_t wait_msec, void *buf, size_t count);
-
-    /** Read exactly count bytes from the stream, using continue_select(). */
-    virtual size_t continue_read(time_t wait_msec, WvBuf &outbuf,
-				 size_t count);
-
     /**
      * Reads up to 'count' bytes of data from the stream into the buffer.
      * Returns the actual amount read.
@@ -183,49 +163,72 @@ public:
      */ 
     virtual size_t uwrite(const void *buf, size_t count)
         { return count; /* basic WvStream doesn't actually do anything! */ }
-    
+
     /**
-     * read up to one line of data from the stream and return a pointer
-     * to the internal buffer containing this line.  If the end-of-line
-     * 'separator' is encountered, it is removed from the string.  If
-     * wait_msec times out before the end of line is found, returns NULL and
-     * the line may be returned next time, or you can read what we have so
-     * far by calling read().
+     * Read up to one line of data from the stream and return a
+     * pointer to the internal buffer containing this line.  If the
+     * end-of-line 'separator' is encountered, it is removed from the
+     * string.  If there is not a full line available, returns
+     * NULL. You can read what we have so far by calling read().
      *
-     * If wait_msec < 0, waits forever for a newline (often a bad idea!)
-     * If wait_msec=0, never waits.  Otherwise, waits up to wait_msec
-     * milliseconds until a newline appears.
+     * Readahead specifies the maximum amount of data that the stream
+     * is allowed to read in one shot.
      *
-     * Readahead specifies the maximum amount of data that the stream is
-     * allowed to read in one shot.
-     *
-     * It is expected that there will be no NULL characters on the line.
-     * 
-     * If uses_continue_select is true, getline() will use continue_select()
-     * rather than select() to wait for its timeout.
-     */
-    char *getline(time_t wait_msec, char separator = '\n',
-		  int readahead = 1024);
-    
-    /**
-     * read up to count characters into buf, up to and including the first
-     * instance of separator.
-     * 
-     * if separator is not found on input before timeout (usual symantics)
-     * or stream close or error, or if count is 0, nothing is placed in buf
-     * and 0 is returned.
-     * 
-     * if your buffer is not large enough for line, call multiple times
-     * until seperator is found at end of buffer to retrieve the entire
+     * It is expected that there will be no NULL characters on the
      * line.
-     * 
-     * Returns the number of characters that were put in buf.
-     * 
-     * If uses_continue_select is true, getline() will use
-     * continue_select() rather than select() to wait for its timeout.
+     *
+     * wait_msec is provided so that legacy code does not break.  But
+     * it really should be 0.
      */
-    size_t read_until(void *buf, size_t count, time_t wait_msec,
-                      char separator);
+    char *getline(time_t wait_msec = 0,
+		  char separator = '\n', int readahead = 1024)
+    {
+	return blocking_getline(wait_msec, separator, readahead);
+    }
+
+    /** Auto-convert int to time_t. */
+    char *getline(int wait_msec,
+		  char separator = '\n', int readahead = 1024)
+    {
+	return getline(time_t(wait_msec), separator, readahead);
+    }
+
+    /** Auto-convert double to time_t. */
+    char *getline(double wait_msec,
+		  char separator = '\n', int readahead = 1024)
+    {
+	return getline(time_t(wait_msec), separator, readahead);
+    }
+
+private:
+    /** We will prohibit someone from calling getline with a char or
+     * bool as the first parameter.  This will attempt to detect dumb
+     * mistakes.
+     */
+    char *getline(char, int i = 0);
+    char *getline(bool, int i = 0);
+public:
+
+    /**
+     * This is a version of getline() that allows you to block for
+     * more data to arrive.
+     *
+     * This should be used carefully, as blocking is generally
+     * unexpected in WvStreams programs.
+     *
+     * If wait_msec < 0, it will wait forever for the 'separator'
+     * (often a bad idea!).  If wait_msed == 0, this is the equivalent
+     * of getline().
+     */
+    char *blocking_getline(time_t wait_msec, int separator = '\n',
+			   int readahead = 1024);
+
+    /**
+     * This is a version of blocking_getline() that uses
+     * continue_select to avoid blocking other streams.
+     */
+    char *continue_getline(time_t wait_msec, int separator = '\n',
+			   int readahead = 1024);
 
     /**
      * force read() to not return any bytes unless 'count' bytes can be
@@ -421,7 +424,14 @@ public:
     bool select(time_t msec_timeout,
 		bool readable, bool writable, bool isex = false)
         { return _select(msec_timeout, readable, writable, isex, false); }
-    
+
+    /**
+     * Use get_select_request() to save the current state of the
+     * selection state of this stream.  That way, you can call
+     * force_select() and undo_force_select() to restore this properly.
+     */
+    IWvStream::SelectRequest get_select_request();
+
     /**
      * Use force_select() to force one or more particular modes (readable,
      * writable, or isexception) to true when selecting on this stream.
@@ -477,8 +487,18 @@ public:
      */
     void setcallback(WvStreamCallback _callfunc, void *_userdata);
         
+    /** Sets a callback to be invoked when the stream is readable. */
+    IWvStreamCallback setreadcallback(IWvStreamCallback _callback);
+
+    /** Sets a callback to be invoked when the stream is writable. */
+    IWvStreamCallback setwritecallback(IWvStreamCallback _callback);
+
+    /** Sets a callback to be invoked when the stream is in exception
+     * state. */
+    IWvStreamCallback setexceptcallback(IWvStreamCallback _callback);
+
     /** Sets a callback to be invoked on close().  */
-    void setclosecallback(WvStreamCallback _callfunc, void *_userdata);
+    IWvStreamCallback setclosecallback(IWvStreamCallback _callback);
 
     /**
      * set the callback function for this stream to an internal routine
@@ -578,13 +598,21 @@ private:
 		 bool readable, bool writable, bool isexcept,
 		 bool forceable);
 
+    void legacy_callback(IWvStream& s);
 
 protected:
+    // FIXME: this one is so bad, I'm not touching it. Quick hack to
+    // make it work anyway.
+    friend class WvHTTPClientProxyStream;
+
     WvDynBuf inbuf, outbuf;
-    WvStreamCallback callfunc, closecb_func;
-    WvCallback<void*,void*> call_ctx;
+
+    WvStreamCallback callfunc;
     void *userdata;
-    void *closecb_data;
+    WvCallback<void*,void*> call_ctx;
+
+    IWvStreamCallback readcb, writecb, exceptcb, closecb;
+
     size_t max_outbuf_size;
     bool outbuf_delayed_flush;
     bool is_auto_flush;
@@ -599,12 +627,7 @@ protected:
     time_t autoclose_time;	// close eventually, even if output is queued
     WvTime alarm_time;          // select() returns true at this time
     WvTime last_alarm_check;    // last time we checked the alarm_remaining
-    bool wvstream_execute_called;
     
-    /** Prevent accidental copying of WvStreams. */
-    WvStream(const WvStream &s) { }
-    WvStream& operator= (const WvStream &s) { return *this; }
-
     /**
      * The callback() function calls execute(), and then calls the user-
      * specified callback if one is defined.  Do not call execute() directly;
@@ -615,10 +638,29 @@ protected:
      * Note: If you override this function in a derived class, you must
      * call the parent execute() yourself from the derived class.
      */
-    virtual void execute();
+    virtual void execute()
+        { }
     
     // every call to select() selects on the globalstream.
     static WvStream *globalstream;
+    
+    // ridiculous hackery for now so that the wvstream unit test can poke
+    // around in the insides of WvStream.  Eventually, inbuf will go away
+    // from the base WvStream class, so nothing like this will be needed.
+#ifdef __WVSTREAM_UNIT_TEST
+public:
+    size_t outbuf_used() 
+        { return outbuf.used(); }
+    size_t inbuf_used()
+        { return inbuf.used(); }
+    void inbuf_putstr(WvStringParm t)
+        { inbuf.putstr(t); }
+#endif
+    
+private:
+    /** Prevent accidental copying of WvStream.  These don't actually exist. */
+    WvStream(const WvStream &s);
+    WvStream& operator= (const WvStream &s);
 };
 
 /**
