@@ -6,19 +6,20 @@
  * on the command line.
  */
 #include "wvtcp.h"
-#include "wvstreamlist.h"
+#include "wvistreamlist.h"
 #include "wvlog.h"
 
-
 int main(int argc, char **argv)
 {
     WvLog err("tcptest", WvLog::Error);
+    err(WvLog::Debug1, "tcptest starting.\n");
+    
     WvTCPConn sock(WvString(argc==2 ? argv[1] : "127.0.0.1:25"));
     
     wvcon->autoforward(sock);
     sock.autoforward(*wvcon);
     
-    WvStreamList l;
+    WvIStreamList l;
     l.add_after(l.tail, wvcon, false);
     l.add_after(l.tail, &sock, false);
     
@@ -32,6 +33,7 @@ int main(int argc, char **argv)
 	err("stdin: %s\n", wvcon->errstr());
     if (!sock.isok() && sock.geterr())
 	err("socket: %s\n", sock.errstr());
+    err(WvLog::Debug1, "tcptest exiting.\n");
     
     return 0;
 }
