@@ -1,10 +1,8 @@
 #include "wvtest.h"
 
-#define private public
-#define protected public
+#define __WVSTREAM_UNIT_TEST 1
 #include "wvstream.h"
-#undef private
-#undef protected
+
 
 #include "wvistreamlist.h"
 #include "wvcont.h"
@@ -34,7 +32,14 @@ public:
     bool yes_writable, block_writes;
     
     CountStream()
-    	{ rcount = wcount = 0; yes_writable = block_writes = false; }
+    { 
+	printf("countstream initializing.\n");
+	rcount = wcount = 0; yes_writable = block_writes = false;
+	int num = geterr();
+	printf("countstream error: %d (%s)\n", num, errstr().cstr());
+	printf("countstream new error: %d\n", geterr());
+    }
+    
     virtual ~CountStream()
     	{ close(); }
     
@@ -47,18 +52,21 @@ public:
     
     virtual size_t uwrite(const void *buf, size_t count)
     {
+	fprintf(stderr, "I'm uwrite! (%d)\n", count);
 	if (block_writes)
 	    return 0;
 	size_t ret = WvStream::uwrite(buf, count);
+	assert(ret == count);
 	wcount += ret;
 	return ret;
     }
     
     virtual bool post_select(SelectInfo &si)
     {
+	printf("countstream post_select\n");
 	int ret = WvStream::post_select(si);
 	if (yes_writable 
-	  && (si.wants.writable || (outbuf.used() && want_to_flush)))
+	  && (si.wants.writable || (outbuf_used() && want_to_flush)))
 	    return true;
 	else
 	    return ret;
@@ -76,7 +84,7 @@ WVTEST_MAIN("buffered read/write")
     WVPASS(!s.isreadable());
     WVPASS(!s.iswritable());
     WVFAIL(s.read(buf, 1024) != 0);
-    WVPASS(s.write(buf, 1024) == 1024);
+    WVPASSEQ(s.write(buf, 1024), 1024);
     WVPASS(!s.iswritable());
     WVPASS(!s.isreadable());
     WVPASS(s.isok());
@@ -103,13 +111,13 @@ WVTEST_MAIN("errors")
     WVPASS(!a.geterr());
     a.seterr(EBUSY);
     WVPASS(!a.isok());
-    WVPASS(a.geterr() == EBUSY);
-    WVPASS(a.errstr() == strerror(EBUSY));
+    WVPASSEQ(a.geterr(), EBUSY);
+    WVPASSEQ(a.errstr(), strerror(EBUSY));
     
     b.seterr("I'm glue!");
     WVPASS(!b.isok());
-    WVPASS(b.geterr() == -1);
-    WVPASS(b.errstr() == "I'm glue!");
+    WVPASSEQ(b.geterr(), -1);
+    WVPASSEQ(b.errstr(), "I'm glue!");
 }
 
 
@@ -134,28 +142,28 @@ WVTEST_MAIN("getline")
     char buf[1024];
     
     WVPASS(!s.isreadable());
-    s.inbuf.putstr("a\n b \r\nline");
+    s.inbuf_putstr("a\n b \r\nline");
     WVPASS(s.isreadable());
     s.noread();
     WVPASS(s.isreadable());
     
-    WVPASS(s.read(buf, 2) == 2);
-    char *line = s.getline(0);
+    WVPASSEQ(s.read(buf, 2), 2);
+    char *line = s.getline();
     WVPASS(line);
     WVPASS(line && !strcmp(line, " b \r"));
-    line = s.getline(0);
+    line = s.getline();
     WVPASS(line);
     WVPASS(line && !strcmp(line, "line"));
-    WVPASS(!s.getline(0));
+    WVPASS(!s.getline());
     
     WvTime t1 = wvtime();
-    WVPASS(!s.getline(500));
+    WVPASS(!s.blocking_getline(500));
     WvTime t2 = wvtime();
     WVPASS(msecdiff(t2, t1) >= 0);
     WVPASS(msecdiff(t2, t1) < 400); // noread().  shouldn't actually wait!
    
     WvStream t;
-    t.inbuf.putstr("tremfodls\nd\ndopple");
+    t.inbuf_putstr("tremfodls\nd\ndopple");
     line = t.getline(0, '\n', 20);
     WVPASS(line && !strcmp(line, "tremfodls"));
     t.close();
@@ -165,9 +173,11 @@ WVTEST_MAIN("getline")
     WVPASS(line && !strcmp(line, "dopple"));
 
     // FIXME: avoid aborting the entire test here on a freezeup!
+#ifndef WIN32
     ::alarm(5); // crash after 5 seconds
-    WVPASS(!s.getline(-1));
+    WVPASS(!s.blocking_getline(-1));
     ::alarm(0);
+#endif 
 }
 
 // more noread/nowrite behaviour
@@ -175,15 +185,15 @@ WVTEST_MAIN("more noread/nowrite")
 {
     WvStream s;
     
-    s.inbuf.putstr("hello");
+    s.inbuf_putstr("hello");
     s.write("yellow");
     WVPASS(s.isok());
     s.nowrite();
     WVPASS(s.isok());
     s.noread();
     WVPASS(s.isok());
-    WVPASS(s.getline(0));
-    WVFAIL(s.getline(-1));
+    WVPASS(s.getline());
+    WVFAIL(s.blocking_getline(-1));
     WVPASS(!s.isok());
 }
 
@@ -194,6 +204,12 @@ static void val_cb(WvStream &s, void *userdata)
 }
 
 
+static void val_icb(int &closeval, WvStream &s)
+{
+    ++closeval;
+}
+
+
 // callback tests
 WVTEST_MAIN("callbacks")
 {
@@ -201,33 +217,47 @@ WVTEST_MAIN("callbacks")
     
     WvStream s;
     s.setcallback(val_cb, &val);
-    s.setclosecallback(val_cb, &closeval);
+    s.setclosecallback(WvBoundCallback<IWvStreamCallback, int&>(&val_icb, closeval));
     
     WVPASS(!val);
     WVPASS(!closeval);
     s.runonce(0);
     WVPASS(!val);
-    s.inbuf.putstr("gah");
+    s.inbuf_putstr("gah");
     s.runonce(0);
-    WVPASS(val == 1); // callback works?
+    WVPASSEQ(val, 1); // callback works?
     s.runonce(0);
-    WVPASS(val == 2); // level triggered?
-    s.getline(0);
-    WVPASS(val == 2); // but not by getline
+    WVPASSEQ(val, 2); // level triggered?
+    s.getline();
+    WVPASSEQ(val, 2); // but not by getline
     WVPASS(!closeval);
-    s.inbuf.putstr("blah!");
+    s.inbuf_putstr("blah!");
     s.nowrite();
     s.noread();
     s.runonce(0);
-    WVPASS(val == 3);
-    WVPASS(s.getline(0));
+    WVPASSEQ(val, 3);
+    WVPASS(s.getline());
     s.runonce(0);
-    WVPASS(val == 3);
-    WVPASS(closeval == 1);
+    WVPASSEQ(val, 3);
+    WVPASSEQ(closeval, 1);
     s.runonce(0);
-    WVPASS(closeval == 1);
+    WVPASSEQ(closeval, 1);
+    s.close();
+    WVPASSEQ(closeval, 1);
+}
+
+
+static void ccb_isok(WvStream &s)
+{
+    WVPASS(!s.isok());
+}
+
+
+WVTEST_MAIN("closecallback-isok")
+{
+    WvStream s;
+    s.setclosecallback(ccb_isok);
     s.close();
-    WVPASS(closeval == 1);
 }
 
 
@@ -238,59 +268,66 @@ WVTEST_MAIN("autoforward and buffers")
     CountStream b;
     int val = 0;
     
+    fprintf(stderr, "a error: '%s'\n", a.errstr().cstr());
+    fprintf(stderr, "b error: '%s'\n", b.errstr().cstr());
+    
     a.autoforward(b);
     b.setcallback(val_cb, &val);
     
-    a.inbuf.putstr("astr");
+    WVPASS(a.isok());
+    WVPASS(b.isok());
+    
+    a.inbuf_putstr("astr");
+    WVPASSEQ(a.inbuf_used(), 4);
     a.runonce(0);
-    WVPASS(!a.inbuf.used());
+    WVPASS(!a.inbuf_used());
     b.runonce(0);
-    WVPASS(val == 0);
-    WVPASS(b.wcount == 4);
+    WVPASSEQ(val, 0);
+    WVPASSEQ(b.wcount, 4);
     a.noautoforward();
-    a.inbuf.putstr("astr2");
+    a.inbuf_putstr("astr2");
     a.runonce(0);
-    WVPASS(a.inbuf.used() == 5);
-    WVPASS(b.wcount == 4);
+    WVPASSEQ(a.inbuf_used(), 5);
+    WVPASSEQ(b.wcount, 4);
     
     // delay_output tests
     a.autoforward(b);
     b.delay_output(true);
     a.runonce(0);
-    WVPASS(!a.inbuf.used());
-    WVPASS(b.wcount == 4);
-    WVPASS(b.outbuf.used() == 5);
+    WVPASS(!a.inbuf_used());
+    WVPASSEQ(b.wcount, 4);
+    WVPASSEQ(b.outbuf_used(), 5);
     b.runonce(0);
-    WVFAIL(!b.outbuf.used());
+    WVFAIL(!b.outbuf_used());
     b.yes_writable = true;
     b.runonce(0);
-    WVFAIL(!b.outbuf.used());
+    WVFAIL(!b.outbuf_used());
     b.flush(0);
-    WVPASS(!b.outbuf.used());
-    WVPASS(b.wcount == 4+5);
+    WVPASS(!b.outbuf_used());
+    WVPASSEQ(b.wcount, 4+5);
     
     // autoforward() has lower precedence than drain()
-    WVPASS(!a.inbuf.used());
-    a.inbuf.putstr("googaa");
+    WVPASS(!a.inbuf_used());
+    a.inbuf_putstr("googaa");
     a.drain();
-    WVPASS(b.wcount == 4+5);
-    WVPASS(!a.inbuf.used());
+    WVPASSEQ(b.wcount, 4+5);
+    WVPASS(!a.inbuf_used());
     
     // queuemin() works
-    a.inbuf.putstr("phleg");
+    a.inbuf_putstr("phleg");
     a.queuemin(2);
     WVPASS(a.isreadable());
     a.queuemin(6);
     WVFAIL(a.isreadable());
     a.drain();
-    WVFAIL(a.getline(0));
+    WVFAIL(a.getline());
     WVFAIL(a.isreadable());
-    WVFAIL(!a.inbuf.used());
-    a.inbuf.putstr("x");
+    WVFAIL(!a.inbuf_used());
+    a.inbuf_putstr("x");
     WVPASS(a.isreadable());
-    WVPASS(a.inbuf.used() == 6);
+    WVPASSEQ(a.inbuf_used(), 6);
     a.drain();
-    WVPASS(!a.inbuf.used());
+    WVPASS(!a.inbuf_used());
 }
 
 
@@ -301,21 +338,21 @@ WVTEST_MAIN("flush_then_close")
     
     s.block_writes = true;
     s.write("abcdefg");
-    WVPASS(s.outbuf.used() == 7);
+    WVPASSEQ(s.outbuf_used(), 7);
     s.flush(0);
-    WVPASS(s.outbuf.used() == 7);
+    WVPASSEQ(s.outbuf_used(), 7);
     s.runonce(0);
-    WVPASS(s.outbuf.used() == 7);
+    WVPASSEQ(s.outbuf_used(), 7);
     s.flush_then_close(20000);
-    WVPASS(s.outbuf.used() == 7);
+    WVPASSEQ(s.outbuf_used(), 7);
     s.runonce(0);
-    WVPASS(s.outbuf.used() == 7);
+    WVPASSEQ(s.outbuf_used(), 7);
     WVPASS(s.isok());
     s.yes_writable = true;
     s.block_writes = false;
     s.runonce(0);
     s.runonce(0);
-    WVPASS(s.outbuf.used() == 0);
+    WVPASSEQ(s.outbuf_used(), 0);
     WVFAIL(s.isok());
 }
 
@@ -329,7 +366,7 @@ WVTEST_MAIN("force_select and globallist")
     
     WVFAIL(s.select(0));
     WVPASS(s.select(0, false, true));
-    s.inbuf.putstr("hello");
+    s.inbuf_putstr("hello");
     WVPASS(s.select(0));
     WVFAIL(s.select(0, false, false));
     s.undo_force_select(true, false, false);
@@ -343,7 +380,7 @@ WVTEST_MAIN("force_select and globallist")
     WvIStreamList::globallist.append(&x, false);
     WVFAIL(s.select(0));
     WVPASS(!val);
-    x.inbuf.putstr("yikes");
+    x.inbuf_putstr("yikes");
     x.force_select(false, true, false);
     WVPASS(!s.select(0));
     WVPASS(val == 1);
@@ -383,7 +420,7 @@ WVTEST_MAIN("continue_select")
     
     a.runonce(0);
     WVPASS(aval == 0);
-    a.inbuf.putstr("gak");
+    a.inbuf_putstr("gak");
     a.runonce(0);
     WVPASS(aval == 1);
     a.runonce(0);
@@ -502,7 +539,7 @@ WVTEST_MAIN("continue_select compatibility with WvCont")
 	s.setcallback(WvBoundCallback<WvStreamCallback, void* >
 		      (&call_wvcont_cb, &cont3), &sval);
 	
-	s.inbuf.putstr("gak");
+	s.inbuf_putstr("gak");
 	WVPASS(sval == 0);
 	s.runonce(0);
 	WVPASS(sval == 1);
@@ -519,57 +556,6 @@ WVTEST_MAIN("continue_select compatibility with WvCont")
 }
 
 
-static void cont_read_cb(WvStream &s, void *userdata)
-{
-    int num = 0;
-    char *line;
-    
-    while (s.isok())
-    {
-	line = s.getline(-1);
-	WVPASS(line);
-	
-	if (!line || !strcmp(line, "end"))
-	    break;
-	WVPASS(++num == atoi(line));
-    }
-    
-    char buf[20];
-    size_t len = s.continue_read(-1, buf, sizeof(buf));
-    WVPASS(len == sizeof(buf));
-}
-
-
-// continue_read() and getline() auto-continuation
-WVTEST_MAIN("continue_read and continuing getline")
-{
-    WvStream s;
-    s.uses_continue_select = true;
-    s.setcallback(cont_read_cb, NULL);
-    
-    s.runonce(0);
-    s.inbuf.putstr("1\n");
-    s.inbuf.putstr("2\n");
-    s.inbuf.putstr("3\n");
-    s.runonce(0);
-    s.runonce(0);
-    s.inbuf.putstr("4");
-    s.runonce(10000);
-    s.inbuf.putstr("\n");
-    s.inbuf.putstr("end\n");
-    s.runonce(0);
-    s.runonce(0);
-    s.inbuf.putstr("1234567890");
-    s.runonce(0);
-    s.inbuf.putstr("123456789");
-    s.runonce(0);
-    s.inbuf.putstr("00");
-    s.runonce(0);
-    WVPASS(s.inbuf.used() == 1);
-    s.terminate_continue_select();
-}
-
-
 static void alarmcall(WvStream &s, void *userdata)
 {
     WVPASS(s.alarm_was_ticking);
@@ -603,10 +589,12 @@ WVTEST_MAIN("alarm")
     WVPASSEQ(val, 1);
     s.runonce(10000);
     WVPASSEQ(val, 2);
+    WVPASSEQ(s.geterr(), 0);
     printf("alarm remaining: %d\n", (int)s.alarm_remaining());
     s.runonce(50);
     WVPASSEQ(val, 2);
-
+    printf("alarm remaining: %d\n", (int)s.alarm_remaining());
+ 
     WvIStreamList l;
     l.append(&s, false, "alarmer");
     
@@ -645,7 +633,7 @@ WVTEST_MAIN("self-redirection")
     WvStream s;
     s.uses_continue_select = true;
     s.setcallback(rcb, NULL);
-    s.inbuf.putstr("x");
+    s.inbuf_putstr("x");
     s.runonce(0);
     s.runonce(0);
     s.runonce(0);
