@@ -12,8 +12,9 @@
  * extended one day when someone cares.
  *
  */
-#include "wvstream.h"
 #include "wvdailyevent.h"
+#include "wvstream.h"
+#include "wvtimeutils.h"
 
 #include <time.h>
 
@@ -22,58 +23,38 @@
 #include <unistd.h>
 #endif
 
+#define NUM_MINS_IN_DAY (24*60)
+#define NUM_SECS_IN_DAY (60*NUM_MINS_IN_DAY)
+
 WvDailyEvent::WvDailyEvent(int _first_hour, int _num_per_day, bool _skip_first)
+    : prev(time(NULL))
 {
-    need_reset = false;
-    prev = time(NULL);
     configure(_first_hour, _num_per_day, _skip_first);
 }
 
 
-// we're ready if now is later than the next scheduled event
+// Compute the next time this stream should select()
 bool WvDailyEvent::pre_select(SelectInfo &si)
 {
-    if (num_per_day && !need_reset)
-    {
-	time_t now = time(NULL), next = next_event();
-	assert(prev);
-	assert(next);
-	assert(prev > 100000);
-	assert(next > 100000);
-	if (now >= next)
-	{
-	    need_reset = true;
-	    prev = next;
-	}
-    }
-    bool ret = WvStream::pre_select(si) || need_reset;
-    // printf("%p ret=%d msd=%d\n", this, ret, si.msec_timeout);
-    return ret;
+    WvTime next(next_event(), 0);
+    if (next)
+	si.msec_timeout = msecdiff(next, wvtime());
+    return WvStream::pre_select(si);
 }
 
 
+// Test to see if the timer has gone off
 bool WvDailyEvent::post_select(SelectInfo& si)
 {
-    return need_reset;
-}
+    bool timer_rang = false;
+    WvTime next(next_event(), 0);
+    if (next < wvtime())
+    {
+	timer_rang = true;
+	prev = next;
+    }
 
-
-void WvDailyEvent::execute()
-{
-    WvStream::execute();
-    reset();
-}
-
-
-void WvDailyEvent::reset()
-{
-    need_reset = false;
-}
-
-
-bool WvDailyEvent::isok() const
-{
-    return true;
+    return WvStream::post_select(si) || timer_rang;
 }
 
 
@@ -83,16 +64,16 @@ void WvDailyEvent::set_num_per_day(int _num_per_day)
     if (num_per_day < 0)
 	num_per_day = 1;
 
-    if (num_per_day > 24*60*60)
-        num_per_day = 24*60*60;
+    if (num_per_day > NUM_SECS_IN_DAY)
+        num_per_day = NUM_SECS_IN_DAY;
         
-    time_t max = num_per_day ? (24*60*60)/num_per_day : 6*60*60;
+    time_t max = num_per_day ? NUM_SECS_IN_DAY/num_per_day : 6*60*60;
     if (max > 6*60*60)
 	max = 6*60*60; // unless that's a very long time, 6 hrs
 
     // don't start until at least one period has gone by
-    not_until = time(NULL) + max;
     prev = time(NULL);
+    not_until = prev + max;
 }
 
 
@@ -103,8 +84,8 @@ void WvDailyEvent::configure(int _first_hour, int _num_per_day, bool _skip_first
 
     // Don't let WvDailyEvents occur more than once a minute. -- use an alarm
     // instead
-    if (_num_per_day > 24*60)
-        _num_per_day = 24*60;
+    if (_num_per_day > NUM_MINS_IN_DAY)
+        _num_per_day = NUM_MINS_IN_DAY;
 
     set_num_per_day(_num_per_day);
 }
@@ -115,18 +96,17 @@ time_t WvDailyEvent::next_event() const
 {
     if (!num_per_day) // disabled
 	return 0;
-    
-    time_t start, next, interval = 24*60*60/num_per_day;
-    struct tm *tm;
-    
+
     assert(prev);
-    start = prev + interval;
     
+    time_t interval = NUM_SECS_IN_DAY/num_per_day;
+    time_t start = prev + interval;
+   
     // find the time to start counting from (up to 24 hours in the past)
-    tm = localtime(&start);
+    struct tm *tm = localtime(&start);
     if (tm->tm_hour < first_hour)
     {
-	start = prev - 24*60*60 + 1; // this time yesterday
+	start = prev - NUM_SECS_IN_DAY + 1; // this time yesterday
 	tm = localtime(&start);
     }
     tm->tm_hour = first_hour; // always start at the given hour
@@ -135,12 +115,13 @@ time_t WvDailyEvent::next_event() const
 
     // find the next event after prev that's a multiple of 'interval'
     // since 'start'
-    next = prev + interval;
+    time_t next = prev + interval;
     if ((next - start)%interval != 0)
 	next = start + (next - start)/interval * interval;
     
     assert(next);
     assert(next > 100000);
+
     while (skip_first && next < not_until)
 	next += interval;
 
