@@ -41,20 +41,18 @@ char *WvLogRcv::loglevels[WvLog::NUM_LOGLEVELS] = {
 
 
 
-WvLog::WvLog(WvStringParm _app, LogLevel _loglevel, const WvLog *par)
-	: app(_app)
+WvLog::WvLog(WvStringParm _app, LogLevel _loglevel)
+    : app(_app), loglevel(_loglevel)
 {
-    parent = par;
-    loglevel = _loglevel;
+//    printf("log: %s create\n", app.cstr());
     num_logs++;
 }
 
 
 WvLog::WvLog(const WvLog &l)
+    : app(l.app), loglevel(l.loglevel)
 {
-    parent = l.parent ? l.parent : &l;
-    app = parent->app;
-    loglevel = parent->loglevel;
+//    printf("log: %s create\n", app.cstr());
     num_logs++;
 }
 
@@ -68,6 +66,8 @@ WvLog::~WvLog()
 	delete default_receiver;
 	default_receiver = NULL;
     }
+//    printf("log: %s delete\n", app.cstr());
+//    printf("num_logs is now %d\n", num_logs);
 }
 
 
@@ -94,10 +94,11 @@ size_t WvLog::uwrite(const void *_buf, size_t len)
 	if (!default_receiver)
 	{
 	    // nobody's listening -- create a receiver on the console
-	    default_receiver = new WvLogConsole(dup(2));
+	    int xfd = dup(2);
+	    default_receiver = new WvLogConsole(xfd);
 	    num_receivers--; // default does not qualify!
 	}
-	default_receiver->log(parent ? parent : this, loglevel,
+	default_receiver->log(app, loglevel,
 			      (const char *)_buf, len);
 	return len;
     }
@@ -113,7 +114,7 @@ size_t WvLog::uwrite(const void *_buf, size_t len)
     for (i.rewind(); i.next(); )
     {
 	WvLogRcvBase &rc = *i;
-	rc.log(parent ? parent : this, loglevel, (const char *)_buf, len);
+	rc.log(app, loglevel, (const char *)_buf, len);
     }
     
     return len;
@@ -141,12 +142,12 @@ WvLogRcvBase::~WvLogRcvBase()
 }
 
 
-const char *WvLogRcvBase::appname(const WvLog *log) const
+const char *WvLogRcvBase::appname(WvStringParm log) const
 {
     if (log)
-	return log->app;
+	return log;
     else
-	return WvString("unknown");
+	return "unknown";
 }
 
 
@@ -181,7 +182,7 @@ void WvLogRcvBase::cleanup_on_fork(pid_t p)
 
 WvLogRcv::WvLogRcv(WvLog::LogLevel _max_level) : custom_levels(5)
 {
-    last_source = NULL;
+    last_source = WvString();
     last_level = WvLog::NUM_LOGLEVELS;
     max_level = _max_level;
     at_newline = true;
@@ -196,7 +197,7 @@ WvLogRcv::~WvLogRcv()
 void WvLogRcv::_make_prefix()
 {
     prefix = WvString("%s<%s>: ",
-        appname(last_source), loglevels[last_level]);
+        last_source, loglevels[last_level]);
     prelen = prefix.len();
 }
 
@@ -225,13 +226,13 @@ static bool my_isprint(char _c)
 }
 
 
-void WvLogRcv::log(const WvLog *source, int _loglevel,
+void WvLogRcv::log(WvStringParm source, int _loglevel,
 			const char *_buf, size_t len)
 {
     WvLog::LogLevel loglevel = (WvLog::LogLevel)_loglevel;
     char hex[5];
     WvLog::LogLevel threshold = max_level;
-    WvString srcname = source->app;
+    WvString srcname(source);
     strlwr(srcname.edit());
 
     Src_LvlDict::Iter i(custom_levels);
@@ -326,7 +327,7 @@ bool WvLogRcv::set_custom_levels(WvString descr)
     WvStringList lst;
     WvStringList::Iter i(lst);
     lst.split(descr, ",= ");
-    if (!lst.count())
+    if (lst.isempty())
         return true;
     WvString src("");
 
