@@ -92,7 +92,7 @@ WvProtoStream::TokenList *WvProtoStream::tokenize()
     while ((t = next_token()) != NULL)
 	tl->append(t, true);
 #if 0 
-    if (logp && log_enable && tl->count())
+    if (logp && log_enable && !tl->isempty())
     {
 	(*logp)("Read: ");
 	TokenList::Iter i(*tl);
@@ -194,7 +194,7 @@ void WvProtoStream::execute()
 {
     WvStreamClone::execute();
     
-    Token *t1 = tokline(getline(0));
+    Token *t1 = tokline(getline());
     
     if (t1)
     {
