@@ -15,14 +15,18 @@
 UniConfDaemonConn::UniConfDaemonConn(WvStream *_s, const UniConf &_root)
     : UniClientConn(_s), root(_root)
 {
+    uses_continue_select = true;
     addcallback();
-    writecmd(EVENT_HELLO, wvtcl_escape("UniConf Server ready."));
+    writecmd(EVENT_HELLO,
+	     spacecat(wvtcl_escape("UniConf Server ready."),
+		      wvtcl_escape(UNICONF_PROTOCOL_VERSION)));
 }
 
 
 UniConfDaemonConn::~UniConfDaemonConn()
 {
     close();
+    terminate_continue_select();
     delcallback();
 }
 
@@ -49,13 +53,11 @@ void UniConfDaemonConn::delcallback()
 void UniConfDaemonConn::execute()
 {
     UniClientConn::execute();
-    
-    for (;;)
-    {
-        UniClientConn::Command command = readcmd();
-        if (command == UniClientConn::NONE)
-            break;
 
+    WvString command_string;
+    UniClientConn::Command command = readcmd(command_string);
+    if (command != UniClientConn::NONE)
+    {
         // parse and execute command
         WvString arg1(readarg());
         WvString arg2(readarg());
@@ -65,7 +67,7 @@ void UniConfDaemonConn::execute()
 	    break;
 	    
 	case UniClientConn::INVALID:
-	    do_malformed();
+	    do_invalid(command_string);
 	    break;
             
 	case UniClientConn::REQ_NOOP:
@@ -74,39 +76,47 @@ void UniConfDaemonConn::execute()
 	    
 	case UniClientConn::REQ_GET:
 	    if (arg1.isnull())
-		do_malformed();
+		do_malformed(command);
 	    else
 		do_get(arg1);
 	    break;
             
 	case UniClientConn::REQ_SET:
 	    if (arg1.isnull() || arg2.isnull())
-		do_malformed();
+		do_malformed(command);
 	    else
 		do_set(arg1, arg2);
 	    break;
 	    
 	case UniClientConn::REQ_REMOVE:
 	    if (arg1.isnull())
-		do_malformed();
+		do_malformed(command);
 	    else
 		do_remove(arg1);
 	    break;
 	    
 	case UniClientConn::REQ_SUBTREE:
 	    if (arg1.isnull())
-		do_malformed();
+		do_malformed(command);
 	    else
 		do_subtree(arg1, arg2.num() == 1);
 	    break;
 	    
 	case UniClientConn::REQ_HASCHILDREN:
 	    if (arg1.isnull())
-		do_malformed();
+		do_malformed(command);
 	    else
 		do_haschildren(arg1);
 	    break;
 	    
+	case UniClientConn::REQ_COMMIT:
+            do_commit();
+	    break;
+	    
+	case UniClientConn::REQ_REFRESH:
+            do_refresh();
+	    break;
+	    
 	case UniClientConn::REQ_QUIT:
 	    do_quit();
 	    break;
@@ -116,16 +126,23 @@ void UniConfDaemonConn::execute()
 	    break;
 	    
 	default:
-	    do_malformed();
+	    do_invalid(command_string);
 	    break;
         }
     }
 }
 
 
-void UniConfDaemonConn::do_malformed()
+void UniConfDaemonConn::do_invalid(WvStringParm c)
 {
-    writefail("malformed request");
+    writefail(WvString("unknown command: %s", c));
+}
+
+
+void UniConfDaemonConn::do_malformed(UniClientConn::Command c)
+{
+    writefail(WvString("malformed request: %s",
+		       UniClientConn::cmdinfos[c].name));
 }
 
 
@@ -165,6 +182,8 @@ void UniConfDaemonConn::do_remove(const UniConfKey &key)
 
 void UniConfDaemonConn::do_subtree(const UniConfKey &key, bool recursive)
 {
+    static int niceness = 0;
+    
     UniConf cfg(root[key]);
     if (cfg.exists())
     {
@@ -172,13 +191,33 @@ void UniConfDaemonConn::do_subtree(const UniConfKey &key, bool recursive)
 	{
 	    UniConf::RecursiveIter it(cfg);
 	    for (it.rewind(); it.next(); )
+	    {
 		writevalue(it->fullkey(cfg), it._value());
+		
+		// the output might be totally gigantic.  Don't hog the
+		// entire daemon while fulfilling it; give up our timeslice
+		// after each entry.
+		if (!isok()) break;
+		if (++niceness > 100)
+		{
+		    niceness = 0;
+		    continue_select(0);
+		}
+	    }
 	}
 	else
 	{
 	    UniConf::Iter it(cfg);
 	    for (it.rewind(); it.next(); )
+	    {
 		writevalue(it->fullkey(cfg), it._value());
+		
+		// the output might be totally gigantic.  Don't hog the
+		// entire daemon while fulfilling it; give up our timeslice
+		// after each entry.
+		if (!isok()) break;
+		continue_select(0);
+	    }
 	}
 	writeok();
     }
@@ -195,6 +234,22 @@ void UniConfDaemonConn::do_haschildren(const UniConfKey &key)
 }
 
 
+void UniConfDaemonConn::do_commit()
+{
+    root.commit();
+    writeok();
+}
+
+
+void UniConfDaemonConn::do_refresh()
+{
+    if (root.refresh())
+        writeok();
+    else
+        writefail();
+}
+
+
 void UniConfDaemonConn::do_quit()
 {
     writeok();
