@@ -1,6 +1,8 @@
+#include "wvautoconf.h"
 #include "uniconfroot.h"
 #include "wvlogrcv.h"
 #include "strutils.h"
+#include "wvstringmask.h"
 #include "wvtclstring.h"
 
 #ifdef _WIN32
@@ -24,13 +26,38 @@ void usage()
 	    "   hdump - list the subkeys/values recursively\n"
 	    "   xdump - list keys/values that match a wildcard\n"
 	    "   del   - delete all subkeys\n"
-	    "   help  - this text\n");
+	    "   help  - this text\n"
+	    "\n"
+	    "You must set the UNICONF environment variable to a valid "
+	    "UniConf moniker.\n"
+	    "\n"
+	    "Report bugs to <" PACKAGE_BUGREPORT ">.\n");
 }
 
 int main(int argc, char **argv)
 {
     WvLogConsole logcon(2, WvLog::Info);
     
+    if (argc < 3)
+    {
+	usage();
+	return 3;
+    }
+    
+    // note: we know cmd and arg1 are non-NULL, but arg2 may be the argv
+    // terminator, which is a NULL.  That has a special meaning for some
+    // commands, like 'set', and is different from the empty string.
+    const char *_cmd = argv[1], *arg1 = argv[2],
+	       *arg2 = argc > 3 ? argv[3] : NULL;
+    WvString cmd(_cmd);
+    strlwr(cmd.edit());
+
+    if (cmd == "help")
+    {
+	usage();
+	return 0;
+    }
+
     const char *confuri = getenv("UNICONF");
     if (!confuri)
     {
@@ -39,12 +66,6 @@ int main(int argc, char **argv)
 	return 2;
     }
     
-    if (argc < 3)
-    {
-	usage();
-	return 3;
-    }
-    
     UniConfRoot cfg(confuri);
     
     if (!cfg.whichmount() || !cfg.whichmount()->isok())
@@ -54,14 +75,7 @@ int main(int argc, char **argv)
 	return 5;
     }
     
-    // note: we know cmd and arg1 are non-NULL, but arg2 may be the argv
-    // terminator, which is a NULL.  That has a special meaning for some
-    // commands, like 'set', and is different from the empty string.
-    const char *_cmd = argv[1], *arg1 = argv[2],
-	       *arg2 = argc > 3 ? argv[3] : NULL;
-    WvString cmd(_cmd);
-    strlwr(cmd.edit());
-    
+    static const WvStringMask nasties("\r\n[]=");
     if (cmd == "get")
     {
 	WvString val = cfg[arg1].getme(arg2);
@@ -100,20 +114,23 @@ int main(int argc, char **argv)
     {
 	UniConf::Iter i(cfg[arg1]);
 	for (i.rewind(); i.next(); )
-	    wvcon->print("%s\n", wvtcl_escape(i->key(), "\r\n"));
+	    wvcon->print("%s\n", wvtcl_escape(i->key(),
+					      WVTCL_NASTY_NEWLINES));
     }
     else if (cmd == "hkeys")
     {
 	UniConf sub(cfg[arg1]);
 	UniConf::RecursiveIter i(sub);
 	for (i.rewind(); i.next(); )
-	    wvcon->print("%s\n", wvtcl_escape(i->fullkey(sub), "\r\n"));
+	    wvcon->print("%s\n", wvtcl_escape(i->fullkey(sub),
+					      WVTCL_NASTY_NEWLINES));
     }
     else if (cmd == "xkeys")
     {
 	UniConf::XIter i(cfg, arg1);
 	for (i.rewind(); i.next(); )
-	    wvcon->print("%s\n", wvtcl_escape(i->fullkey(cfg), "\r\n"));
+	    wvcon->print("%s\n", wvtcl_escape(i->fullkey(cfg),
+					      WVTCL_NASTY_NEWLINES));
     }
     else if (cmd == "dump")
     {
@@ -122,8 +139,8 @@ int main(int argc, char **argv)
 	UniConf::Iter i(cfg[arg1]);
 	for (i.rewind(); i.next(); )
 	    wvcon->print("%s = %s\n",
-			 wvtcl_escape(i->key(), "\r\n[]="),
-			 wvtcl_escape(i->getme(""), "\r\n[]="));
+			 wvtcl_escape(i->key(), nasties),
+			 wvtcl_escape(i->getme(""), nasties));
     }
     else if (cmd == "hdump")
     {
@@ -133,8 +150,8 @@ int main(int argc, char **argv)
 	UniConf::RecursiveIter i(sub);
 	for (i.rewind(); i.next(); )
 	    wvcon->print("%s = %s\n",
-			 wvtcl_escape(i->fullkey(sub), "\r\n[]="),
-			 wvtcl_escape(i->getme(""), "\r\n[]="));
+			 wvtcl_escape(i->fullkey(sub), nasties),
+			 wvtcl_escape(i->getme(""), nasties));
     }
     else if (cmd == "xdump")
     {
@@ -143,8 +160,8 @@ int main(int argc, char **argv)
 	UniConf::XIter i(cfg, arg1);
 	for (i.rewind(); i.next(); )
 	    wvcon->print("%s = %s\n",
-			 wvtcl_escape(i->fullkey(cfg), "\r\n[]="),
-			 wvtcl_escape(i->getme(""), "\r\n[]="));
+			 wvtcl_escape(i->fullkey(cfg), nasties),
+			 wvtcl_escape(i->getme(""), nasties));
     }
     else if (cmd == "del")
     {
@@ -152,11 +169,6 @@ int main(int argc, char **argv)
 	sub.remove();
 	cfg.commit();
     }
-    else if (cmd == "help")
-    {
-	usage();
-	return 5;
-    }
     else
     {
 	fprintf(stderr, "%s: unknown command '%s'!\n", argv[0], _cmd);
