@@ -1,6 +1,6 @@
 #include "uniconfroot.h"
 #include "wvcallback.h"
-#include "wvstreamlist.h"
+#include "wvistreamlist.h"
 
 /**
  * This test is mostly for using with the uniconf daemon to see what
@@ -18,7 +18,7 @@ public:
 
     void cb(int a, const UniConf &moo, const UniConfKey &goo)
         { fprintf(stderr, "Moo... '%d' - goo = %s (%s)\n", a,
-            goo.printable().cstr(), moo[goo].get().cstr()); moo.set("whee"); }
+            goo.printable().cstr(), moo[goo].getme().cstr()); moo.setme("whee"); }
 };
 
 int main()
@@ -42,7 +42,7 @@ int main()
 		  WvBoundCallback<UniConfCallback, int>(&thing, &foo::cb, 3));
 
  
-    WvStreamList l;
+    WvIStreamList l;
 
     while (true)
     {
