@@ -10,18 +10,16 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+#include "wvlinkerhack.h"
+
+WV_LINK(UniDefGen);
+
+
 // if 'obj' is non-NULL and is a UniConfGen, wrap that; otherwise wrap the
 // given moniker.
-static IUniConfGen *creator(WvStringParm s, IObject *obj, void *)
+static IUniConfGen *creator(WvStringParm s)
 {
-    IUniConfGen *gen = NULL;
-
-    if (obj)
-        gen = mutate<IUniConfGen>(obj);
-    if (!gen)
-        gen = wvcreate<IUniConfGen>(s);
-
-    return new UniDefGen(gen);
+    return new UniDefGen(wvcreate<IUniConfGen>(s));
 }
 
 // this name is too confusing.  We should deprecate it.
@@ -106,6 +104,7 @@ WvString UniDefGen::replacewildcard(const UniConfKey &key,
         }
     }
 
+
     // pull the literal from that segment num of the key
     return key.segment(loc-1);
 }
@@ -123,7 +122,7 @@ UniConfKey UniDefGen::keymap(const UniConfKey &key)
     UniConfKey result = finddefault(key, p, q);
     if (!result.numsegments())
 	result = key;
-    // wvcon->print("mapping '%s' -> '%s'\n", key, result);
+    // fprintf(stderr, "mapping '%s' -> '%s'\n", key.cstr(), result.cstr());
     return result;
 }
 
@@ -131,8 +130,9 @@ UniConfKey UniDefGen::keymap(const UniConfKey &key)
 WvString UniDefGen::get(const UniConfKey &key)
 {
     UniConfKey defkey = keymap(key);
+
     return replacewildcard(key, defkey,
-			   inner() ? inner()->get(defkey) : WvString());
+ 			   inner() ? inner()->get(defkey) : WvString());
 }
 
 
