@@ -8,8 +8,12 @@
 #include "wvbuf.h"
 #include "wvtclstring.h"
 #include "wvmoniker.h"
+#include "wvlinkerhack.h"
 
-static IUniConfGen *creator(WvStringParm s, IObject *obj, void *)
+WV_LINK(UniSubtreeGen);
+
+
+static IUniConfGen *creator(WvStringParm s)
 {
     WvConstInPlaceBuf buf(s, s.len());
     WvString one(wvtcl_getword(buf)), two(wvtcl_getword(buf));
@@ -17,12 +21,7 @@ static IUniConfGen *creator(WvStringParm s, IObject *obj, void *)
     if (!one) one = "";
     if (!two) two = "";
     
-    if (obj)
-	return new UniSubtreeGen(mutate<IUniConfGen>(obj), one);
-    else
-    {
-	return new UniSubtreeGen(wvcreate<IUniConfGen>(one), two);
-    }
+    return new UniSubtreeGen(wvcreate<IUniConfGen>(one), two);
 }
 
 static WvMoniker<IUniConfGen> subtreereg("subtree", creator);
@@ -37,7 +36,12 @@ UniSubtreeGen::UniSubtreeGen(IUniConfGen *gen, const UniConfKey &_subkey)
 
 UniConfKey UniSubtreeGen::keymap(const UniConfKey &key)
 {
-    return UniConfKey(subkey, key);
+    if (key == UniConfKey())
+        return subkey;
+    else if (subkey == UniConfKey())
+        return key;
+    else
+        return UniConfKey(subkey, key);
 }
 
 UniConfKey UniSubtreeGen::reversekeymap(const UniConfKey &key)
