@@ -12,7 +12,9 @@
 #include "wvbuf.h"
 #include "wvbase64.h"
 #include "strutils.h"
-#include <execinfo.h>
+#ifndef _WIN32
+#include <execinfo.h> // FIXME: add a WvCrash feature for explicit dumps
+#endif
 
 #ifdef _WIN32
 #define ETIMEDOUT WSAETIMEDOUT
@@ -46,6 +48,19 @@ WvHttpStream::WvHttpStream(const WvIPPortAddr &_remaddr, WvStringParm _username,
 
 WvHttpStream::~WvHttpStream()
 {
+    log(WvLog::Debug2, "Deleting.\n");
+
+#ifndef _WIN32
+    void* trace[10];
+    int count = backtrace(trace, sizeof(trace)/sizeof(trace[0]));
+    char** tracedump = backtrace_symbols(trace, count);
+    log(WvLog::Debug, "TRACE");
+    for (int i = 0; i < count; ++i)
+        log(WvLog::Debug, ":%s", tracedump[i]);
+    log(WvLog::Debug, "\n");
+    free(tracedump);
+#endif
+
     if (geterr())
         log("Error was: %s\n", errstr());
     close();
@@ -54,6 +69,19 @@ WvHttpStream::~WvHttpStream()
 
 void WvHttpStream::close()
 {
+    log("close called\n");
+    
+#ifndef _WIN32
+    void *trace[10];
+    int count = backtrace(trace, sizeof(trace)/sizeof(trace[0]));
+    char** tracedump = backtrace_symbols(trace, count);
+    log(WvLog::Debug, "TRACE");
+    for (int i = 0; i < count; ++i)
+        log(WvLog::Debug, ":%s", tracedump[i]);
+    log(WvLog::Debug, "\n");
+    free(tracedump);
+#endif
+
     // assume pipelining is broken if we're closing without doing at least
     // one successful pipelining test and a following non-test request.
     if (enable_pipelining && max_requests > 1
@@ -80,7 +108,11 @@ void WvHttpStream::close()
     }
     waiting_urls.zap();
     if (curl)
+    {
+        log("curl is %s\n", curl->url);
         doneurl();
+    }
+    log("close done\n");
 }
 
 
@@ -370,7 +402,7 @@ void WvHttpStream::execute()
     if (!curl)
     {
         // in the header section
-        line = getline(0);
+        line = getline();
         if (line)
         {
             line = trim_string(line);
@@ -538,7 +570,7 @@ void WvHttpStream::execute()
     }
     else if (encoding == Chunked && !bytes_remaining)
     {
-        line = getline(0);
+        line = getline();
         if (line)
         {
             line = trim_string(line);
