@@ -28,3 +28,8 @@ unsigned WvHash(const int &i)
 }
 
 
+unsigned WvHash(const void *p)
+{
+    return reinterpret_cast<unsigned long>(p);
+}
+
