@@ -172,7 +172,7 @@ WvTask *WvTaskMan::start(WvStringParm name,
 	if (i().stacksize >= stacksize)
 	{
 	    t = &i();
-	    i.link->auto_free = false;
+	    i.set_autofree(false);
 	    i.unlink();
 	    t->recycled = false;
 	    t->start(name, func, userdata);
